 /**----------------------------------------------------
 * File:      KeyInputManager.java
 * Package:   at.hoertler.svgeditor.common
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * This class represents the global KeyListener.
 * SVGElements can be registered as receiver and the KeyInputManager
 * will forward all inputs.
 */
package at.hoertler.svgeditor.common;

import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.display.controls.Canvas;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import at.hoertler.svgeditor.options.OptionManager;
import at.hoertler.svgeditor.svg.elements.SVGElement;
import at.hoertler.svgeditor.svg.elements.SVGText;
import java.awt.KeyEventDispatcher;
import java.awt.KeyboardFocusManager;
import java.awt.Point;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;


public class KeyInputManager
        implements KeyListener, KeyEventDispatcher {

  private OptionManager optionManager = null;
  private static SVGElement receiver = null;
  private static Canvas canvas = null;
  
  public KeyInputManager(OptionManager om) {
    this.optionManager = om;
    String typedText = "";
    KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
        manager.addKeyEventDispatcher(this);
  }
  
  public static boolean registerKeyInputReceiver(SVGElement elem) {
    if(receiver != null && 
      !elem.isTextEditMode() && 
      (elem.getClass().isAssignableFrom(SVGText.class) && !elem.isEditMode())) {
      return false;
    }
    receiver = elem;
    return true;
  }
  
  public static boolean removeKeyInputReceiver(SVGElement elem) {

    if(receiver!=null && receiver.equals(elem)) {
      receiver = null;
      return true;
    }
    return false;
  }
  
  public static void registerCanvas(Canvas c) {
    canvas = c;
  }
  
  @Override
  public void keyTyped(KeyEvent e) {
    if(this.isReceiverOK()) {
      if(!e.isActionKey() && !e.isControlDown() && receiver.getClass().isAssignableFrom(SVGText.class) && receiver.isEditMode()) {
        receiver.setText("");
        receiver.setMode(SVGElement.Mode.TextEdit);
      }
      receiver.handleKeyTyped(e);
      e.consume();
      return;
    }
  }
  
  @Override
  public void keyPressed(KeyEvent e) {
    if(e.getKeyCode() == KeyEvent.VK_SHIFT || e.getKeyCode() == KeyEvent.VK_ALT) {
      canvas.updateMouse(e); 
    }
    if(this.isReceiverOK()) {
      if(e.getKeyCode() == KeyEvent.VK_ENTER && !e.isShiftDown()) {
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
              ActionPerformedEvent.Action.switchToSelection));
        this.receiver.setMode(SVGElement.Mode.Edit);
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
              ActionPerformedEvent.Action.repaint));
        this.receiver = null;
      } else {
        receiver.handleKeyPressed(e);
      }
    } else if(e.getKeyCode() == KeyEvent.VK_BACK_SPACE) {
      ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
              ActionPerformedEvent.Action.delete));
      this.receiver = null;
    }
    ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                  ActionPerformedEvent.Action.repaint));

    ActionPerformedEvent ev = null;
    switch(e.getKeyCode()) {
      case KeyEvent.VK_UP:
        ev = new ActionPerformedEvent(ActionPerformedEvent.Action.moveUp);
        if(e.isAltDown())
          ev.setValue(new Point(0,-this.optionManager.getGridWideness()));
        break;
      case KeyEvent.VK_DOWN:
        ev = new ActionPerformedEvent(ActionPerformedEvent.Action.moveDown);
        if(e.isAltDown())
          ev.setValue(new Point(0,this.optionManager.getGridWideness()));
        break;
      case KeyEvent.VK_LEFT:
        ev = new ActionPerformedEvent(ActionPerformedEvent.Action.moveLeft);
        if(e.isAltDown())
          ev.setValue(new Point(-this.optionManager.getGridWideness(),0));
        break;
      case KeyEvent.VK_RIGHT:
        ev = new ActionPerformedEvent(ActionPerformedEvent.Action.moveRight);
        if(e.isAltDown())
          ev.setValue(new Point(this.optionManager.getGridWideness(),0));
        break;
    }
    if(ev != null)
      ActionManager.fireActionPerformedEvent(ev);
  }
  
  @Override
  public void keyReleased(KeyEvent e) {
    if(e.getKeyCode() == KeyEvent.VK_SHIFT || e.getKeyCode() == KeyEvent.VK_ALT) {
      //canvas.mouseMoved(new MouseEvent(canvas, MouseEvent.MOUSE_MOVED, System.nanoTime(), 0, canvas.getMousePosition().x, canvas.getMousePosition().y, 0, false));
      canvas.updateMouse(e); 
     }
    if(this.isReceiverOK()) {
      receiver.handleKeyReleased(e);
     
      
      e.consume();
      ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
              ActionPerformedEvent.Action.repaint));
      return;
    }
  }

  @Override
  public boolean dispatchKeyEvent(KeyEvent e) {

    if (e.getID() == KeyEvent.KEY_PRESSED) {
      this.keyPressed(e);
    } else if (e.getID() == KeyEvent.KEY_RELEASED) {
      this.keyReleased(e);
    } else if (e.getID() == KeyEvent.KEY_TYPED) {
      this.keyTyped(e);
    }
    return false;
  }
  
  private boolean isReceiverOK() {
    if(this.receiver==null || !this.receiver.isSelected()) {
      //removeKeyInputReceiver();
      this.receiver = null;
      return false;
    }
    return true;
  }
}
