 /**----------------------------------------------------
 * File:      ElementManager.java
 * Package:   at.hoertler.svgeditor.data
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * The ElementManager keeps all SVGElements and is the interface to the 
 * HistoryManager.
 * This class is used to
 *    - execute actions to the elements list, 
 *    - generate PNG, PDF, SVG,
 *    - handle the clipboard,
 *    - ...
 */
package at.hoertler.svgeditor.data;

import at.hoertler.svgeditor.Reader;
import at.hoertler.svgeditor.action.ActionManager;
import at.hoertler.svgeditor.common.HelperFunction;
import at.hoertler.svgeditor.common.SVGImporter;
import at.hoertler.svgeditor.event.ActionPerformedEvent;
import at.hoertler.svgeditor.event.ActionPerformedEvent.Alignment;
import at.hoertler.svgeditor.event.ActionPerformedListener;
import at.hoertler.svgeditor.event.ElementManagerStateEvent;
import at.hoertler.svgeditor.event.ElementManagerStateListener;
import at.hoertler.svgeditor.event.OptionChangedEvent;
import at.hoertler.svgeditor.event.OptionChangedListener;
import at.hoertler.svgeditor.options.OptionManager;
import at.hoertler.svgeditor.svg.elements.SVGComposite;
import at.hoertler.svgeditor.svg.elements.SVGElement;
import at.hoertler.svgeditor.svg.elements.SVGText;
import com.itextpdf.awt.FontMapper;
import com.itextpdf.awt.PdfGraphics2D;
import com.itextpdf.text.Document;
import com.itextpdf.text.DocumentException;
import com.itextpdf.text.FontFactory;
import com.itextpdf.text.Meta;
import com.itextpdf.text.pdf.BaseFont;
import com.itextpdf.text.pdf.PdfContentByte;
import com.itextpdf.text.pdf.PdfWriter;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.awt.print.PrinterJob;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JOptionPane;


public class ElementManager
        implements OptionChangedListener,
                   ActionPerformedListener {
  private HistoryManager historyManager = null;
  private Vector<SVGElement> elements;
  
  private LinkedList<ElementManagerStateListener> listener = null;
  private File openedFile = null;
  private int gridWideness = 1;
  private boolean inCompositeEditMode = false;
  private LinkedList<SVGComposite> composites;
  /**
   * Creates an instance of ElementManager.
   */
  public ElementManager(OptionManager om) {
    this.historyManager = new HistoryManager();
    this.elements = new Vector<SVGElement>();//historyManager.getElements();
    this.composites = new LinkedList<SVGComposite>();
    om.addOptionChangedListener(this);
    this.gridWideness = om.getGridWideness();
    this.listener = new LinkedList<ElementManagerStateListener>();
  }
  
  /**
   * Creates a new History-Entry.
   */
  public void CreateNewHistoryPoint() {
    if(!this.isInCompositeEditMode()) {
    this.historyManager.CreateNewHistoryPoint(this.getAllElements());
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canRedo, this.historyManager.redoPossible()));
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canUndo, this.historyManager.undoPossible()));
    }
  }

  /**
   * Adds an listener to the ElementManager.
   *
   * @param l ElementManagerStateListener
   */
  public void addElementManagerStateListener(ElementManagerStateListener l) {
    listener.add(l);
  }

  /**
   * Removes an listener from the ElementManager.
   *
   * @param l ElementManagerStateListener
   */
  public void removeElementManagerStateListener(ElementManagerStateListener l) {
    listener.remove(l);
  }

  /**
   * Fires the given ElementManagerStateEvent to all listeners.
   *
   * @param e ElementManagerStateEvent that the listeners should recognize.
   */
  public void fireElementManagerStateEvent(ElementManagerStateEvent e) {
    for (ElementManagerStateListener l : this.listener) {
      l.elementManagerStateChanged(e);
    }
  }

  /**
   * @return the inCompositeEditMode
   */
  public boolean isInCompositeEditMode() {
    return inCompositeEditMode;
  }

  /**
   * @param inCompositeEditMode the inCompositeEditMode to set
   */
  public void setInCompositeEditMode(boolean inCompositeEditMode) {
    this.inCompositeEditMode = inCompositeEditMode;
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.isInCompositeEditMode, inCompositeEditMode));
  }

  /**
   * Set File and import all containing elements
   * @param f File containing SVG-Code.
   */
  public void openFile(File f) {
    this.setElements(SVGImporter.Import(f));
    if(!this.getAllElements().isEmpty()) {
      this.openedFile = f;
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, true, this.openedFile.getName()));
    }
  }
  /**
   * Inserts a new SVGElement at the end of the vector.
   *
   * @param element SVGElement to be inserted.
   *
   * @return true if the add process was successful.
   */
  public boolean addElement(SVGElement element) {
    if (!element.validate()) {
      return false;
    }
    this.deselectAll();
    //if(element.isExtendedEdit())
    //element.select();
    boolean done = getAllElements().add(element);

    if (done && getAllElements().size() > 0) {
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasElements, true));
      if(element.isSelected())
        this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasSelectedElements, true));
    }
    if(done && !this.isInCompositeEditMode())// && element.getClass().isAssignableFrom(SVGText.class))
      this.CreateNewHistoryPoint();
    return done;
  }
  
    /**
   * Sets all SVGElements.
   *
   * @param elems Elements to be set.
   */
  public void setElements(Vector<SVGElement> elems) {
    if(this.isInCompositeEditMode()) {
      this.composites.getFirst().setElements(elems);
    } else
      this.elements = elems;
    

    if (getAllElements().size() > 0)
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasElements, true));
    
    this.historyManager.setElements(this.elements);
  }

  /**
   * Gets all SVGElements.
   *
   * @return A vector with all SVGElements.
   */
  public Vector<SVGElement> getAllElements() {
    if(this.isInCompositeEditMode())
      return this.composites.getFirst().getElements();
    return this.elements;
    //return this.historyManager.getElements();
  }

  /**
   * Gets all selected SVGElements.
   *
   * @return a vector with all selected SVGElements
   */
  public Vector<SVGElement> getSelectedElements() {
    Vector<SVGElement> elems = new Vector<SVGElement>();
    for (SVGElement e : this.getAllElements()) {
      if (e.isSelected()) {
        elems.add(e);
      }
    }
    return elems;
  }

  /**
   * Sets an SVGElement selected.
   *
   * @param element Element to be selected.
   *
   * @return true if the operation was successful.
   */
  public boolean setSelectedElement(SVGElement element) {

    for (SVGElement e : this.getAllElements()) {
      if (e.equals(element)) {
        e.select();
        this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.hasSelectedElements, true));
        return true;
      }
    }
    return false;
  }

  /**
   * Sets the given SVGElements selected.
   *
   * @param elements Vector with SVGElements to be selected.
   *
   * @return true if the operation was successful.
   */
  public boolean setSelectedElements(Vector<SVGElement> elements) {
    boolean ret = false;
    this.deselectAll();
    for (SVGElement e : this.getAllElements()) {
      for (SVGElement e1 : elements) {
        if (e.equals(e1)) {
          e.select();
          ret = true;
        }
      }
    }
    if (ret) {
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasSelectedElements, true));
    }
    return ret;
  }

  /**
   * Method deselectAll.
   *
   * @return boolean
   */
  public boolean deselectAll() {
    boolean ret = false;
    for (SVGElement e : this.getAllElements()) {
      if (e.isSelected()) {
        e.deSelect();
        ret = true;
      }
    }
    if (ret) {
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasSelectedElements, false));
    }
    return ret;
  }

  /**
   * Deletes all selected SVGElements.
   *
   * @return true if the operation was successful.
   */
  public boolean deleteSelectedElements() {
    if (this.getSelectedElements().isEmpty()) {
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasElements, false));
      return false;
    }

    Vector<SVGElement> toRemove = new Vector<SVGElement>();
    for (SVGElement elem : this.getSelectedElements())
      toRemove.add(elem);

    for (SVGElement elem : toRemove)
      this.getAllElements().remove(elem);
    
    if(!toRemove.isEmpty()) this.historyManager.CreateNewHistoryPoint(this.getAllElements());
    
    if (this.getAllElements().size() == 0) {
      this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.hasElements, false));
    }
    return true;
  }

  /**
   * Duplicates all selected SVGElements.
   *
   * @return true if the operation was successful.
   */
  public Vector<SVGElement> duplicateSelectedElements() {
    Vector<SVGElement> newElems = new Vector<SVGElement>();
    Vector<SVGElement> dupl = new Vector<SVGElement>();
    SVGElement clone = null;
    if (this.getSelectedElements().isEmpty()) {
      return dupl;
    }
    
    for(int i = 0; i<this.getAllElements().size(); i++) {
      SVGElement elem = this.getAllElements().get(i);
      if(elem.isSelected()) {
        elem.deSelect();
        clone = elem.clone();
        clone.select();
        dupl.add(clone);
        newElems.add(elem);
        newElems.add(clone);
        //this.getAllElements().add(++i, clone);
      } else {
        newElems.add(elem);
      }
    }
    /*
    for (SVGElement elem : this.getSelectedElements()) {
      elem.deSelect();
      dupl.add(elem.clone());
    }

    for (SVGElement elem : dupl) {
      elem.select();
      this.getAllElements().add(elem);
    }
*/
    this.setElements(newElems);
    for(SVGElement e : dupl)
      e.select();
    return dupl;
  }

  /**
   * Gets an ListIterator for all SVGElements.
   *
   * @return An ListIterator.
   */
  public ListIterator<SVGElement> getIterator() {
    return this.getAllElements().listIterator();
  }

  /**
   * Performs an HitTest for the given point with a standard tolerance of 2
   * pixel in each direction.
   *
   * @param point Point that should be tested.
   *
   * @return The element that is hit or null.
   */
  public SVGElement getHitElement(Point point) {
    int tolerance = 2;
    return this.getHitElement(point, tolerance);
  }

  /**
   * Performs an HitTest for the given point with a given tolerance.
   *
   * @param point     Point that should be tested.
   * @param tolerance Tolerance in pixel to test with an rectangle around the
   *                  given point.
   *
   * @return The element that is hit or null.
   */
  public SVGElement getHitElement(Point point, int tolerance) {
    ListIterator<SVGElement> it = this.getAllElements().listIterator(this.getAllElements().
            size());
    SVGElement e = null;

    while (it.hasPrevious()) {
      e = it.previous();
      tolerance += e.getLineStrength() / 2;
      if (e.hit(new Point(point.x - tolerance, point.y - tolerance),
                new Dimension(tolerance * 2, tolerance * 2))) {
        return e;
      }
    }
    return null;
  }

  /**
   * Performs an HitTest for the given rectangle. Elements are hit when they
   * intersects with the rectangle.
   *
   * @param rect Rectangle which should be used to detect hit-elements.
   *
   * @return Vector of SVGElements that have been hit.
   */
  public Vector<SVGElement> getHitElements(Rectangle rect) {
    Vector<SVGElement> hitElems = new Vector<SVGElement>();
    ListIterator<SVGElement> it = this.getAllElements().listIterator(this.getAllElements().
            size());
    SVGElement e = null;
    while (it.hasPrevious()) {
      e = it.previous();
      if (e.hit(rect.getLocation(), rect.getSize())) {
        hitElems.add(e);
      }
    }
    return hitElems;
  }

  /**
   * Performs an HitTest for the given rectangle. Elements are only hit when
   * they are fully situated within the rectangle.
   *
   * @param rect Rectangle which should be used to detect hit-elements.
   *
   * @return Vector of SVGElements that have been hit.
   */
  public Vector<SVGElement> getTotallyHitElements(Rectangle rect) {
    Vector<SVGElement> hitElems = new Vector<SVGElement>();
    ListIterator<SVGElement> it = this.getAllElements().listIterator(this.getAllElements().
            size());
    SVGElement e = null;
    while (it.hasPrevious()) {
      e = it.previous();
      if (rect.contains(e.getBoundingBox())) {
        hitElems.add(e);
      }
    }
    return hitElems;
  }

  /**
   * Gets the index of a hit handle.
   *
   * @param p Point to be checkt for hitting a handle.
   *
   * @return -1 if no handle has been hit or >=0 if a handle has been found.
   */
  public int hitHandle(Point p) {
    int index = -1;
    for (SVGElement e : this.getAllElements()) {
      if (e.isSelected()) {
        index = e.getHandleIndex(p);
        if (index != -1) {
          return index;
        }
      }
    }
    return -1;
  }

  /**
   * Method moveSelectedElements.
   *
   * @param point Point
   */
  public void moveSelectedElements(Point point) {
    for (SVGElement elem : this.getSelectedElements()) {
      elem.translate(point);
    }
  }
  
  public Vector<SVGElement> normalizeElements(Vector<SVGElement> elems, int margin) {
    Dimension d = new Dimension(0, 0);
    Point p = new Point(Integer.MAX_VALUE,Integer.MAX_VALUE);
    Vector<SVGElement> export = new Vector<SVGElement>();
    for (SVGElement elem : elems) {
      export.add(elem.clone());
      p.x = (p.x > elem.getBoundingBox().x) ? elem.getBoundingBox().x : p.x;
      p.y = (p.y > elem.getBoundingBox().y) ? elem.getBoundingBox().y : p.y;
      
    }
    p.x = margin-p.x;
    p.y = margin-p.y;
    d.width = d.width+margin+p.x;
    d.height = d.height+margin+p.y;
    
    for (SVGElement elem : export)
      elem.translate(p);

    return export;
  }
  
  public PrintableBufferedImage generatePrintableBufferedImage(Vector<SVGElement> elems, int margin) {
    //int margin = 10;
    Dimension d = new Dimension(0, 0);
    
    Vector<SVGElement> export = (elems==null || elems.isEmpty()) ? this.getAllElements() : elems;
    export = this.normalizeElements(export, margin);
    
    for (SVGElement elem : export) {
      d.height = (d.height < elem.getBoundingBox().height+elem.getBoundingBox().y) ? elem.getBoundingBox().height+elem.getBoundingBox().y : d.height;
      d.width = (d.width < elem.getBoundingBox().width+elem.getBoundingBox().x) ? elem.getBoundingBox().width+elem.getBoundingBox().x : d.width;
    }
    d.width = d.width + margin;
    d.height = d.height + margin;
    d.width *= 8;
    d.height *= 8;
    PrintableBufferedImage img = new PrintableBufferedImage(d.width, d.height,BufferedImage.TYPE_INT_ARGB);
    Graphics2D g = (Graphics2D)img.createGraphics();
    for (SVGElement elem : export)
      elem.paint(g,8);

    img.elements = this.normalizeElements(this.getAllElements(), this.gridWideness*3);
    return img;
    /*int w = img.getWidth()/4;
    int h = img.getHeight()/4;
    PrintableBufferedImage after = new PrintableBufferedImage(w, h, BufferedImage.TYPE_INT_ARGB);
    AffineTransform at = new AffineTransform();
    at.scale(0.25, 0.25);
    AffineTransformOp scaleOp = 
       new AffineTransformOp(at, AffineTransformOp.TYPE_BILINEAR);
    after = (PrintableBufferedImage)scaleOp.filter(img, after);
    return after;*/
  }

  public String generateSVGString(Vector<SVGElement> elems, int margin) {
    Dimension d = new Dimension(0, 0);
    
    Vector<SVGElement> export = (elems==null || elems.isEmpty()) ? this.getAllElements() : elems;
    export = this.normalizeElements(export, margin);
    StringBuilder sb = new StringBuilder();
    sb.append("  <!-- Created with SVGEditor by Philipp Hoertler -->\n");
    
    for (SVGElement elem : export) {
      d.height = (d.height < elem.getBoundingBox().height+elem.getBoundingBox().y) ? elem.getBoundingBox().height+elem.getBoundingBox().y : d.height;
      d.width = (d.width < elem.getBoundingBox().width+elem.getBoundingBox().x) ? elem.getBoundingBox().width+elem.getBoundingBox().x : d.width;
      sb.append(elem.exportToSVG());
    }
    d.width = d.width + margin*2;
    d.height = d.height + margin*2;
    sb.append("</svg>");
    sb.insert(0, "<svg width=\"" + (d.width) + "\" height=\"" + (d.height) + "\" xmlns=\"http://www.w3.org/2000/svg\">\n");
    return sb.toString();
  }

  @Override
  public void optionChanged(OptionChangedEvent e) {
    switch (e.getOption()) {
      case lineStrength:
      case lineColor:
      case fillColor:
      case font:
      case text:
      case textColor:
      case textSize:
      case textAlignment:
      case textOption:
        for (SVGElement elem : this.getSelectedElements()) {
          elem.setValueForOption(e.getOption(), e.getValue());
        }
        this.CreateNewHistoryPoint();
        break;
      case gridWideness:
        /*int grid = (int) e.getValue();
        Point tmp = null;
        for (SVGElement elem : this.getAllElements()) {
          tmp = elem.getHandles().get(0);
          if (tmp.x % grid != 0 || tmp.y % grid != 0) {
            elem.translate(this.getGridTranslationPoint(tmp, grid), true);
          }
          tmp = null;
        }*/
        this.gridWideness = (int) e.getValue();
        break;
      case zoomFactor:
        /*
         * for (SVGElement elem : this.getAllElements()) {
         * elem.setValueForOption(e.getOption(), e.getValue()); }
         */
        break;
      default:
        break;
    }
  }

  private Point getGridTranslationPoint(Point p, int grid) {
    Point ret = new Point();
    int x = p.x;
    int y = p.y;
    if (x % grid > ((double) grid / 2)) {
      ret.x = (grid - (x % grid));
    } else {
      ret.x = -(x % grid);
    }

    if (y % grid > ((double) grid / 2)) {
      ret.y = (grid - (y % grid));
    } else {
      ret.y = -(y % grid);
    }
    return ret;
  }

  @Override
  public void actionPerformed(ActionPerformedEvent e) {
    switch (e.getAction()) {
      case createHistoryPoint:
        this.CreateNewHistoryPoint();
        break;
      case closeWindow:
        ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(ActionPerformedEvent.Action.newImage));
        if(this.getAllElements().isEmpty())
          System.exit(0);
        break;
      case newImage:
        if(!this.getAllElements().isEmpty()) {
          int ret = JOptionPane.showOptionDialog(null,
                                                 Reader.getPropertyValueAsString(Reader.StringProperty.newFile_SaveFirst_Text), 
                                                 Reader.getPropertyValueAsString(Reader.StringProperty.newFile_SaveFirst_Title), 
                                                 JOptionPane.YES_NO_OPTION, 
                                                 JOptionPane.QUESTION_MESSAGE,
                                                 null,
                                                 null,
                                                 null);
          if(ret==JOptionPane.YES_OPTION)
            this.actionPerformed(new ActionPerformedEvent(ActionPerformedEvent.Action.saveImage));
          if(ret!=JOptionPane.CLOSED_OPTION) {
            this.selectAll();
            this.elements.clear();
            this.composites.clear();
            this.setInCompositeEditMode(false);
            this.historyManager.clearHistory();
          }
          
        }
        this.openedFile = null;
        this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, false, null));
        
        break;
      case insertImage:
        File f = SVGImporter.LoadOpenFile();
        if(f!=null) {
          this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, true, f.getName()));
          Vector<SVGElement> elems = SVGImporter.Import(f);
          SVGComposite cmp = new SVGComposite();
          for(SVGElement elem : elems)
            cmp.addElement(elem);
          this.addElement(cmp);
        }
        break;
      case openImage:
        this.openedFile = SVGImporter.LoadOpenFile();
        if(this.openedFile!=null) {
          this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, true, this.openedFile.getName()));
          this.setElements(SVGImporter.Import(this.openedFile));
        }
        break;
      case saveImageAs:
        if(this.getAllElements().isEmpty()) {
          JOptionPane.showMessageDialog(null, 
                                        Reader.getPropertyValueAsString(Reader.StringProperty.saveError_NoElements_Text), 
                                        Reader.getPropertyValueAsString(Reader.StringProperty.saveError_Title), 
                                        JOptionPane.ERROR_MESSAGE);
          return;
        }
        //if(this.openedFile == null) {
          this.openedFile = SVGImporter.LoadSaveFile();
          this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, true, this.openedFile.getName()));
          this.handleSaveImage(this.openedFile);
        //} else {
        //  this.handleSaveImage(SVGImporter.LoadSaveFile());
        //}
        break;
      case saveImage:
        if(this.getAllElements().isEmpty()) {
          JOptionPane.showMessageDialog(null, 
                                        Reader.getPropertyValueAsString(Reader.StringProperty.saveError_NoElements_Text), 
                                        Reader.getPropertyValueAsString(Reader.StringProperty.saveError_Title), 
                                        JOptionPane.ERROR_MESSAGE);
          return;
        }
        
        if(this.openedFile == null) {
          this.openedFile = SVGImporter.LoadSaveFile();
          if(this.openedFile!=null) {
          this.fireElementManagerStateEvent(new ElementManagerStateEvent(
                ElementManagerStateEvent.Type.openedImage, true, this.openedFile.getName()));
          } else
            return;
        }
        this.handleSaveImage(this.openedFile);

        
        break;
      case deselectAll:
        this.deselectAll();
        break;
      case invertSelection:
        this.invertSelection();
        break;
      case selectAll:
        this.selectAll();
        break;
      case copy:
        this.handleCopy();
        break;
      case paste:
        this.handlePaste();
        break;
      case cut:
        this.handleCut();
        break;
      case delete:
        this.deleteSelectedElements();
        break;
      case duplicate:
        this.duplicateSelectedElements();
        if(this.gridWideness==1)
          this.moveSelectedElements(new Point(10,10));
        else
          this.moveSelectedElements(new Point(this.gridWideness,this.gridWideness));
        break;
      case group:
        this.handleGroup();
        break;
      case ungroup:
        this.handleUngroup();
        break;
      case leaveGroup:
        this.endEditComposite();
        break;
      case enterGroup:
        this.startEditComposite((SVGComposite)e.getValue());
        break;
      case undo:
        this.handleUndo();
        break;
      case redo:
        this.handleRedo();
        break;
      case align:
        this.handleAlign(e.getAlignment());
        break;
      case moveLayerUp:
        this.handleMoveLayerUp();
        break;
      case moveLayerDown:
        this.handleMoveLayerDown();
        break;
      case bringToFront:
        this.handleBringToFront();
        break;
      case sendToBack:
        this.handleSendToBack();
        break;
      case moveUp:
        if(e.getValue()!=null)
          this.moveSelectedElements((Point)e.getValue());
        else
          this.moveSelectedElements(new Point(0,-1));
        break;
      case moveDown:
        if(e.getValue()!=null)
          this.moveSelectedElements((Point)e.getValue());
        else
          this.moveSelectedElements(new Point(0,1));
        break;
      case moveLeft:
        if(e.getValue()!=null)
          this.moveSelectedElements((Point)e.getValue());
        else
          this.moveSelectedElements(new Point(-1,0));
        break;
      case moveRight:
        if(e.getValue()!=null)
          this.moveSelectedElements((Point)e.getValue());
        else
          this.moveSelectedElements(new Point(1,0));
        break;
      case print:
        try {
          PrinterJob pjob = PrinterJob.getPrinterJob();
          if (pjob.printDialog() == false)
            return;
          pjob.setPrintable(this.generatePrintableBufferedImage(null, 10));
          pjob.print();
        } catch (Exception ex) {
          ex.printStackTrace();
        }
        break;
    }
  }

  private void handleCopy() {
    int margin = 10;
    Vector<SVGElement> normalized = this.normalizeElements(this.getSelectedElements(), margin);

    Dimension d = new Dimension(0, 0);
    for (SVGElement elem : normalized) {
      d.height = (d.height < elem.getBoundingBox().height+elem.getBoundingBox().y) ? elem.getBoundingBox().height+elem.getBoundingBox().y : d.height;
      d.width = (d.width < elem.getBoundingBox().width+elem.getBoundingBox().x) ? elem.getBoundingBox().width+elem.getBoundingBox().x : d.width;
    }
    d.width = d.width + margin*2;
    d.height = d.height + margin*2;
    BufferedImage img = new BufferedImage(d.width, d.height,BufferedImage.TYPE_INT_ARGB);
    Graphics2D g = (Graphics2D)img.createGraphics();
    g.transform(new AffineTransform(0.125, 0, 0, 0.125, 0, 0));

    for (SVGElement elem : normalized) {
      elem.paint(g,8);
    }
    String svg = this.generateSVGString(this.getSelectedElements(), this.gridWideness*3);
    
    try {
      //File file = File.createTempFile("tmpTestFile", ".pdf");

      //this.saveAsPDF(file, this.gridWideness*3, this.getSelectedElements());

      // This Byte[] contains the PNG-Image with the SVG-Metadata !!
      byte[] bArr = HelperFunction.writeCustomData(img, "SVGCode", svg);
      File f = File.createTempFile("tmpTestPNG", ".png");
      FileOutputStream fos = new FileOutputStream(f);
      fos.write(bArr);
      fos.close(); 
      ImageSelection trans = new ImageSelection(null, f, svg);
      Clipboard c = Toolkit.getDefaultToolkit().getSystemClipboard();
      c.setContents(trans, null);

    } catch (Exception ex) {
      Logger.getLogger(ElementManager.class.getName()).log(Level.SEVERE, null,
                                                           ex);
    }
  }
  
    private void handleCut() {
    this.handleCopy();
    this.deleteSelectedElements();
  }

  private void handlePaste() {
    try {
      Clipboard sysClip = Toolkit.getDefaultToolkit().getSystemClipboard();
      Transferable transfer = sysClip.getContents( null );
     
      if(transfer.isDataFlavorSupported(DataFlavor.stringFlavor)) {
        String data = (String) transfer.getTransferData( DataFlavor.stringFlavor );
        Vector<SVGElement> selElems = this.getSelectedElements();
        if(!data.startsWith("<svg")) {
          for(SVGElement elem : selElems) {
            if(elem.isTextEditMode())
              elem.setText(elem.getText() + data);
            else if (elem.getClass().isAssignableFrom(SVGText.class) && elem.isEditMode())
              elem.setText(data);
          }
          return;
        }
        FileWriter fstream = null;
        File f = File.createTempFile("input", ".svg");
        fstream = new FileWriter(f);
        BufferedWriter out = new BufferedWriter(fstream);
        out.write(data);
        out.close();
        for(SVGElement e : SVGImporter.Import(f))
          this.addElement(e);
      } else if(transfer.isDataFlavorSupported(DataFlavor.javaFileListFlavor)) {
        List<File> fileList = (List<File>)transfer.getTransferData(DataFlavor.javaFileListFlavor);
        if(!fileList.isEmpty() && ((List<File>)fileList).get(0).getName().toLowerCase().endsWith(".png")) {
          File f = ((List<File>)fileList).get(0);
          String svgInfo = HelperFunction.getPNGMetaData(f, "SVGCode");
          for(SVGElement e : SVGImporter.Import(f))
            this.addElement(e);
        } else if(!fileList.isEmpty() && ((List<File>)fileList).get(0).getName().toLowerCase().endsWith(".pdf")) {
          File f = ((List<File>)fileList).get(0);
          for(SVGElement e : SVGImporter.Import(f))
            this.addElement(e);
        }
      }/* else if(transfer.isDataFlavorSupported(DataFlavor.imageFlavor)) {
        Object img = transfer.getTransferData(DataFlavor.imageFlavor);
        ClipboardTransferable trans = (ClipboardTransferable)img;
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ObjectOutputStream os = new ObjectOutputStream(out);
        os.writeObject(img);
        System.out.println(HelperFunction.readCustomData(out.toByteArray(), "SVGCode"));
      }*/
    } catch (IOException ex) {
      Logger.getLogger(ElementManager.class.getName()).log(Level.SEVERE,
                                                           null,
                                                           ex);
    } catch (UnsupportedFlavorException ex) {
      Logger.getLogger(ElementManager.class.getName()).log(Level.SEVERE, null,
                                                           ex);
    }
    
  }

  private void handleGroup() {
    if (this.getSelectedElements().size() > 1) {
      SVGComposite comp = new SVGComposite();
      for (SVGElement elem : this.getSelectedElements()) {
        elem.deSelect();
        comp.addElement(elem);
        this.getAllElements().remove(elem);
      }
      comp.select();
      this.addElement(comp);
    }
  }

  private void handleUngroup() {
    if (this.getSelectedElements().size() == 1 && this.getSelectedElements().
            get(0).getClass().isAssignableFrom(SVGComposite.class)) {
      SVGComposite comp = (SVGComposite) this.getSelectedElements().get(0);
      this.getAllElements().remove(comp);
      for (SVGElement elem : comp.getElements()) {
        this.getAllElements().add(elem);
        elem.select();
      }
    }
  }

  private void handleUndo() {
    this.historyManager.undo();
    this.elements = this.historyManager.getHistoryPoint();
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canRedo, this.historyManager.redoPossible()));
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canUndo, this.historyManager.undoPossible()));
  }

  private void handleRedo() {
    this.historyManager.redo();
    this.elements = this.historyManager.getHistoryPoint();
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canRedo, this.historyManager.redoPossible()));
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
              ElementManagerStateEvent.Type.canUndo, this.historyManager.undoPossible()));
  }

  private void handleAlign(Alignment alignment) {
    Point p1 = null;
    Point p2 = null;
    Point transl = null;
    Point tmp = null;
    Vector<SVGElement> elems = this.getSelectedElements();
    if(elems.isEmpty())
      elems = this.getAllElements();
    this.CreateNewHistoryPoint();
    switch (alignment) {
      case none:
        
        break;
      case left_up:
        p1 = calcUpperLeftCorner();
        
        for(SVGElement e : elems) {
          transl = HelperFunction.calculateTranslationPoint(e.getBoundingBox().getLocation(), p1);
          e.translate(transl);
        }
        break;
      case center_up:
        p1 = calcUpperLeftCorner();
        p2 = calcUpperRightCorner();
        p1.x += (p2.x-p1.x)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.x += e.getBoundingBox().width/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case right_up:
        p1 = calcUpperRightCorner();
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.x += e.getBoundingBox().width;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case left_center:
        p1 = calcUpperLeftCorner();
        p2 = calcLowerLeftCorner();
        p1.y += (p2.y-p1.y)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case center_center:
        p1 = calcUpperLeftCorner();
        p2 = calcLowerRightCorner();
        p1.x += (p2.x-p1.x)/2;
        p1.y += (p2.y-p1.y)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height/2;
          tmp.x += e.getBoundingBox().width/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case right_center:
        p1 = calcUpperRightCorner();
        p2 = calcLowerRightCorner();
        p1.y += (p2.y-p1.y)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height/2;
          tmp.x += e.getBoundingBox().width;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case left_bottom:
        p1 = calcLowerLeftCorner();

        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case center_bottom:
        p1 = calcLowerLeftCorner();
        p2 = calcLowerRightCorner();
        p1.x += (p2.x-p1.x)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height;
          tmp.x += e.getBoundingBox().width/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case right_bottom:
        p1 = calcLowerRightCorner();
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height;
          tmp.x += e.getBoundingBox().width;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          e.translate(transl);
        }
        break;
      case left:
        p1 = calcUpperLeftCorner();
        
        for(SVGElement e : elems) {
          transl = HelperFunction.calculateTranslationPoint(e.getBoundingBox().getLocation(), p1);
          transl.y = 0;
          e.translate(transl);
        }
        break;
      case center_h:
        p1 = calcLowerLeftCorner();
        p2 = calcLowerRightCorner();
        p1.x += (p2.x-p1.x)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.x += e.getBoundingBox().width/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          transl.y = 0;
          e.translate(transl);
        }
        break;
      case right:
        p1 = calcLowerRightCorner();
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.x += e.getBoundingBox().width;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          transl.y = 0;
          e.translate(transl);
        }
        break;
      case top:
        p1 = calcUpperLeftCorner();
        
        for(SVGElement e : elems) {
          transl = HelperFunction.calculateTranslationPoint(e.getBoundingBox().getLocation(), p1);
          transl.x = 0;
          e.translate(transl);
        }
        break;
      case center_v:
        p1 = calcUpperLeftCorner();
        p2 = calcLowerRightCorner();
        p1.y += (p2.y-p1.y)/2;
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height/2;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          transl.x = 0;
          e.translate(transl);
        }
        break;
      case bottom:
        p1 = calcLowerRightCorner();
        
        for(SVGElement e : elems) {
          tmp = e.getBoundingBox().getLocation();
          tmp.y += e.getBoundingBox().height;
          transl = HelperFunction.calculateTranslationPoint(tmp, p1);
          transl.x = 0;
          e.translate(transl);
        }
        break;
    }
    this.CreateNewHistoryPoint();
  }
  
  private Point calcUpperLeftCorner() {
    Point p = new Point(Integer.MAX_VALUE, Integer.MAX_VALUE);
    Vector<SVGElement> elems = this.getSelectedElements();
    if(elems.isEmpty())
      elems = this.getAllElements();
    for(SVGElement e : elems) {
      Rectangle r = e.getBoundingBox();
      if(p.x>r.x)
        p.x = r.x;
      if(p.y>r.y)
        p.y = r.y;
    }
    return p;
  }
  
  private Point calcLowerLeftCorner() {
    Point p = new Point(Integer.MAX_VALUE, 0);
    Vector<SVGElement> elems = this.getSelectedElements();
    if(elems.isEmpty())
      elems = this.getAllElements();
    for(SVGElement e : elems) {
      Rectangle r = e.getBoundingBox();
      if(p.x>r.x)
        p.x = r.x;
      if(p.y<(r.y+r.height))
        p.y = r.y+r.height;
    }
    return p;
  }
  
  private Point calcUpperRightCorner() {
    Point p = new Point(0,Integer.MAX_VALUE);
    Vector<SVGElement> elems = this.getSelectedElements();
    if(elems.isEmpty())
      elems = this.getAllElements();
    for(SVGElement e : elems) {
      Rectangle r = e.getBoundingBox();
      if(p.x<(r.x+r.width))
        p.x = r.x+r.width;
      if(p.y>r.y)
        p.y = r.y;
    }
    return p;
  }
  
  private Point calcLowerRightCorner() {
    Point p = new Point(0,0);
    Vector<SVGElement> elems = this.getSelectedElements();
    if(elems.isEmpty())
      elems = this.getAllElements();
    for(SVGElement e : elems) {
      Rectangle r = e.getBoundingBox();
      if(p.x<(r.x+r.width))
        p.x = r.x+r.width;
      if(p.y<(r.y+r.height))
        p.y = r.y+r.height;
    }
    return p;
  }

  private void handleMoveLayerUp() {
    throw new UnsupportedOperationException("Not yet implemented");
  }

  private void handleMoveLayerDown() {
    throw new UnsupportedOperationException("Not yet implemented");
  }

  private void handleBringToFront() {
    Vector<SVGElement> selected = this.getSelectedElements();
    this.getAllElements().removeAll(selected);
    this.getAllElements().addAll(this.getAllElements().size(), selected);
  }

  private void handleSendToBack() {
    Vector<SVGElement> selected = this.getSelectedElements();
    this.getAllElements().removeAll(selected);
    this.getAllElements().addAll(0, selected);
  }

  private void invertSelection() {
    for (SVGElement elem : this.getAllElements()) {
      if (elem.isSelected()) {
        elem.deSelect();
      } else {
        elem.select();
      }
    }
    this.fireElementManagerStateEvent(new ElementManagerStateEvent(
            ElementManagerStateEvent.Type.hasSelectedElements, this.
            getSelectedElements().size() > 0));
  }

  private void selectAll() {
    this.setSelectedElements(this.getAllElements());
  }
  
  public void startEditComposite(SVGComposite comp) {
    this.setInCompositeEditMode(true);
    comp.setInCompositeEditMode(true);
    this.composites.addFirst(comp);
  }
  
  public void endEditComposite() {
    if(this.isInCompositeEditMode() && !this.composites.isEmpty()) {
      SVGComposite cmp = this.composites.removeFirst();
      cmp.setInCompositeEditMode(false);
      if(this.composites.isEmpty()) {
        this.setInCompositeEditMode(false);
        
        if(cmp.getElements().size()==1)
          ActionManager.fireActionPerformedEvent(new ActionPerformedEvent(
                  ActionPerformedEvent.Action.ungroup));
      }
    }
  }

  private void handleSaveImage(File file) {
    if(file!=null) {
      try {
        int margin = this.gridWideness*3;
        if(file.equals(this.openedFile))
          this.openedFile = file;
        if(file.getName().endsWith(".svg")) {
          FileWriter fstream = null;
          fstream = new FileWriter(file);
          BufferedWriter out = new BufferedWriter(fstream);
          out.write(this.generateSVGString(null, margin));
          out.close();
        } else if(file.getName().endsWith(".png")) {
          BufferedImage bufImg = this.generatePrintableBufferedImage(null,margin);
          //ImageIO.write(bufImg, "png", file);
          FileOutputStream fos = new FileOutputStream(file);
          fos.write(HelperFunction.writeCustomData(bufImg, "SVGCode", this.generateSVGString(null, margin)));
          fos.close(); 
        } else if(file.getName().endsWith(".pdf")) {

          this.saveAsPDF(file, margin, null);
          
        }
      } catch (IOException ex) {
        Logger.getLogger(ElementManager.class.getName()).log(Level.SEVERE, null, ex);
      } catch (Exception ex) {
        Logger.getLogger(ElementManager.class.getName()).log(Level.SEVERE, null, ex);
      }
    }
  }
  
  private void saveAsPDF(File file, int margin, Vector<SVGElement> elems) throws FileNotFoundException {
    if(elems==null)
      elems = this.getAllElements();
    try {
      PrintableBufferedImage img = this.generatePrintableBufferedImage(null,margin);
      Document document = new Document(new com.itextpdf.text.Rectangle(img.getWidth()/8, img.getHeight()/8),0,0,0,0);
      PdfWriter writer = PdfWriter.getInstance(document, new FileOutputStream(file));
      document.open();
      PdfContentByte cb = writer.getDirectContent();
      
      cb.concatCTM(1, 0, 0, 1, 0, 0);
      if(FontFactory.getRegisteredFonts().size() < 20);
        FontFactory.registerDirectories();

      FontMapper fm = new FontMapper() {
        public BaseFont awtToPdf(Font font) {
          com.itextpdf.text.Font f = FontFactory.getFont(font.getName(), BaseFont.CP1252, BaseFont.EMBEDDED);
          BaseFont bf = f.getBaseFont();
          return bf;
        }

        public Font pdfToAwt(BaseFont font, int size) {
          return null;
        }
      };
      Graphics2D g2 = new PdfGraphics2D(cb, img.getWidth()/8, img.getHeight()/8, fm);
      g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
                          RenderingHints.VALUE_ANTIALIAS_ON);
      g2.setRenderingHint(RenderingHints.KEY_RENDERING, 
                          RenderingHints.VALUE_RENDER_QUALITY);
      g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
      g2.setRenderingHint(RenderingHints.KEY_STROKE_CONTROL, RenderingHints.VALUE_STROKE_NORMALIZE);
     //g2.transform(new AffineTransform(0.125, 0, 0, 0.125, 0, 0));
      g2.transform(new AffineTransform(0.125, 0, 0, 0.125, 0, 0));

      for(SVGElement elem : this.normalizeElements(elems, margin)) {
        elem.paint(g2,8);
      }
        
      g2.dispose();

      document.add(new Meta("SvgCode", this.generateSVGString(elems, margin)));

      document.close();
      writer.close();
    } catch (DocumentException ex) {
      Logger.getLogger(ElementManager.class.getName()).
              log(Level.SEVERE, null, ex);
    }
  }
}
