 /**----------------------------------------------------
 * File:      MagneticGrid.java
 * Package:   at.hoertler.svgeditor.display.controls
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 *
 * The magnetic grid is used to adapt the mousepositions to
 * an imaginary grid with an adjustable gap width and 
 * renders a visual grid whose gap width is based on the magnetic one.
 * The visual grid is cached in a bufferd image for performance reasons.
 */
package at.hoertler.svgeditor.display.controls;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.image.BufferedImage;


public class MagneticGrid {

  public enum GridType {
    cross_small, cross_big, dot;
  }
  private int width = 0;
  private int height = 0;
  private int magneticGapSize = 4;
  private int displayedGapSize = 20;
  private int zoom = 1;
  private GridType type = GridType.cross_small;
  private boolean magnetic = true;
  private boolean gridDisplayed = true;
  private boolean magneticGridDisplayed = false;
  private BufferedImage buffer;

  public MagneticGrid(int width, int height) {
    this.setWidth(width);
    this.setHeight(height);
  }

  public void paint(Graphics2D g) {
    if (this.buffer != null && this.isGridDisplayed())
      g.drawImage(this.buffer, 0, 0, null);
  }

  public void updateBuffer() {
    System.out.println(getHeight() + " " + getWidth());
    if (getHeight() == 0 || getWidth() == 0) {
      if (buffer != null)
        buffer.flush();

      buffer = null;
    } else {
      if (buffer != null) {
        buffer.flush();
        buffer = null;
      }

      buffer = new BufferedImage(getWidth(), getHeight(),
                                 BufferedImage.TYPE_INT_ARGB);

      Graphics2D g = (Graphics2D) buffer.getGraphics();
      g.setStroke(new BasicStroke(0, BasicStroke.CAP_BUTT, BasicStroke.JOIN_MITER));
      g.setRenderingHint(RenderingHints.KEY_RENDERING,
                         RenderingHints.VALUE_RENDER_QUALITY);
      g.setRenderingHint(RenderingHints.KEY_ALPHA_INTERPOLATION,
                         RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
      
      
      
      
      // Paint Raster
      int grid = 0;
      if(this.isMagneticGridDisplayed()) {
        grid = this.getMagneticGapSize();
        g.setColor(new Color(150, 150, 255));
        for (int i = 0; i < this.getWidth(); i = i + grid * this.getZoom()) {
          for (int j = 0; j < this.getHeight(); j = j + grid * this.getZoom()) {
            g.drawOval(i-1, j-1, 1, 1);
          }
        } 
      }
      g.setColor(new Color(235, 235, 255));
      grid = (int) (Math.ceil(this.getDisplayedGapSize() / (double) this.getMagneticGapSize()) * this.getMagneticGapSize());
      
      int crossSize = 0;
      switch(this.getType()) {
        case cross_small:
          crossSize = grid * 10 / 100;
        case cross_big:
          if(crossSize==0)
            crossSize = grid * 30 / 100;
          for (int i = 0; i < this.getWidth(); i = i + grid * this.getZoom()) {
            for (int j = 0; j < this.getHeight(); j = j + grid * this.getZoom()) {
              g.drawLine(i, j - crossSize, i, j + crossSize);
              g.drawLine(i - crossSize, j, i + crossSize, j);
            }
          }
          break;
        case dot:
          g.setColor(new Color(150, 150, 255));
          for (int i = 0; i < this.getWidth(); i = i + grid * this.getZoom()) {
            for (int j = 0; j < this.getHeight(); j = j + grid * this.getZoom()) {
              g.drawOval(i, j, 1, 1);
            }
          }
          break;
      }

      
    }
  }

  /**
   * Transform a point to another "stick" to the magnetic grid.
   *
   * @param pt The point to transform.
   *
   * @return The transformed point or if there is no magnetic grid, a clone of
   *         the given point.
   */
  public Point getTransformedPointToGrid(Point pt) {
    if (isMagnetic()) {
      Point point = (Point) pt.clone();
      double modulo = this.getMagneticGapSize();
      double x = point.x;
      double y = point.y;
      int base = (int) (((int) (x / modulo)) * modulo);

      if (x > modulo)
        x = x % ((int) modulo);

      double res = modulo - x;
      x = base;

      if (res < modulo / 2.)
        x += modulo;

      point.x = (int) x;
      base = (int) (((int) (point.y / modulo)) * modulo);

      if (y > modulo)
        y = y % ((int) modulo);

      res = modulo - y;
      y = base;

      if (res < modulo / 2.)
        y += modulo;

      point.y = (int) y;
      return point;
    }
    return (Point) pt.clone();
  }

  /**
   * @return the width
   */
  public int getWidth() {
    return width;
  }

  /**
   * @param width the width to set
   */
  public void setWidth(int width) {
    if(width != this.getWidth()) {
      this.width = width;
      this.updateBuffer();
    }
  }

  /**
   * @return the height
   */
  public int getHeight() {
    return height;
  }

  /**
   * @param height the height to set
   */
  public void setHeight(int height) {
    if(height != this.getHeight()) {
      this.height = height;
      this.updateBuffer();
    }
  }

  /**
   * @return the magneticGapSize
   */
  public int getMagneticGapSize() {
    if(this.isMagnetic())
      return magneticGapSize;
    return 1;
  }

  /**
   * @param magneticGapSize the magneticGapSize to set
   */
  public void setMagneticGapSize(int magneticGapSize) {
    this.magneticGapSize = magneticGapSize;
    this.updateBuffer();
  }

  /**
   * @return the displayedGapSize
   */
  public int getDisplayedGapSize() {
    return displayedGapSize;
  }

  /**
   * @param displayedGapSize the displayedGapSize to set
   */
  public void setDisplayedGapSize(int displayedGapSize) {
    this.displayedGapSize = displayedGapSize;
    this.updateBuffer();
  }

  /**
   * @return the zoom
   */
  public int getZoom() {
    return zoom;
  }

  /**
   * @param zoom the zoom to set
   */
  public void setZoom(int zoom) {
    this.zoom = zoom;
    this.updateBuffer();
  }

  /**
   * @return the type
   */
  public GridType getType() {
    return type;
  }

  /**
   * @param type the type to set
   */
  public void setType(GridType type) {
    this.type = type;
  }

  /**
   * @return the magnetic
   */
  public boolean isMagnetic() {
    return magnetic;
  }

  /**
   * @param magnetic the magnetic to set
   */
  public void setMagnetic(boolean magnetic) {
    this.magnetic = magnetic;
  }

  /**
   * @return the gridDisplayed
   */
  public boolean isGridDisplayed() {
    return gridDisplayed;
  }

  /**
   * @param gridDisplayed the gridDisplayed to set
   */
  public void setGridDisplayed(boolean gridDisplayed) {
    this.gridDisplayed = gridDisplayed;
    this.updateBuffer();
  }

  /**
   * @return the magneticGridDisplayed
   */
  public boolean isMagneticGridDisplayed() {
    return magneticGridDisplayed;
  }

  /**
   * @param magneticGridDisplayed the magneticGridDisplayed to set
   */
  public void setMagneticGridDisplayed(boolean magneticGridDisplayed) {
    this.magneticGridDisplayed = magneticGridDisplayed;
    this.updateBuffer();
  }
}
