/**----------------------------------------------------
 * File:      ToolBarManager.java
 * Package:   at.hoertler.svgeditor.display.toolbar
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * The ToolBarManager is used to build up the complete ToolBar.
 * New Buttons must be inserted here.
 */
package at.hoertler.svgeditor.display.toolbar;

import at.hoertler.svgeditor.Editor;
import at.hoertler.svgeditor.Reader;
import at.hoertler.svgeditor.event.OptionChangedEvent;
import at.hoertler.svgeditor.event.ToolBarFocusListener;
import at.hoertler.svgeditor.options.OptionManager;
import at.hoertler.svgeditor.svg.elements.SVGEllipse;
import at.hoertler.svgeditor.svg.elements.SVGLine;
import at.hoertler.svgeditor.svg.elements.SVGPolygone;
import at.hoertler.svgeditor.svg.elements.SVGRectangle;
import at.hoertler.svgeditor.svg.elements.SVGSelection;
import at.hoertler.svgeditor.svg.elements.SVGText;
import at.hoertler.svgeditor.svg.elements.SVGTriangle;
import java.util.HashMap;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;


public class ToolBarManager implements ToolBarFocusListener {
    private Editor editor = null;
    private ToolBarButton activeButton = null;
    private ToolBarButton previousButton = null;
    private ToolBarSelectionButton selectionButton = null;
    private JToolBar toolBar = null;
    private OptionManager optionManager = null;
    
    public ToolBarManager(Editor aThis) {
        this.editor = aThis;
        this.toolBar = new JToolBar();
        this.toolBar.setOrientation(SwingConstants.VERTICAL);
        this.toolBar.setFocusable(false);
        this.toolBar.setFloatable(false);
        this.optionManager = this.editor.getOptionManager();
        

        this.init();
    }
    
    private void init() {
        ToolBarSelectionButton selBut = new ToolBarSelectionButton(Reader.getPropertyValueAsString(Reader.StringProperty.select), this, new SVGSelection());
        this.setActiveButton(selBut);
        this.selectionButton = selBut;
        this.toolBar.add(selBut);

        // Arrow
        SVGLine line = new SVGLine();
        line.setHasArrowAtFront(true);
        ToolBarDrawingButton drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawArrow), this, this.optionManager, line.getInstance());
        this.toolBar.add(drawBut);
        
        // Line
        line.setHasArrowAtFront(false);
        line.setHasArrowAtEnd(false);
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawLine), this, this.optionManager, line.getInstance());
        this.toolBar.add(drawBut);
        
        // Triangle
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawTriangle), this, this.optionManager, new SVGTriangle());
        this.toolBar.add(drawBut);
        
        // Polygone
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawPoly), this, this.optionManager, new SVGPolygone());
        this.toolBar.add(drawBut);
        
        // Rectangle
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawRect), this, this.optionManager, new SVGRectangle());
        this.toolBar.add(drawBut);
        
        // Rounded Rectangle
        SVGRectangle rect = new SVGRectangle();
        rect.setType(SVGRectangle.Type.RoundedRectangle);
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawRoundedRect), this, this.optionManager, rect);
        this.toolBar.add(drawBut);
        
        // Round Sided Rectangle
        rect = new SVGRectangle();
        rect.setType(SVGRectangle.Type.RoundSidedRectangle);
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawRoundedRect), this, this.optionManager, rect);
        this.toolBar.add(drawBut);
        
        // Ellipse
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawEllipse), this, this.optionManager, new SVGEllipse());
        this.toolBar.add(drawBut);
        
        // Text
        drawBut = new ToolBarDrawingButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawText), this, this.optionManager, new SVGText());
        this.toolBar.add(drawBut);
        
        // Line Strength
        ToolBarOptionButton opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.chooseLineStrength), this, this.optionManager, new LineStrengthOptionPopUp(OptionChangedEvent.Option.lineStrength, this.optionManager.getLineStrength()));
        this.toolBar.add(opBut);
        
        // Line Color
        opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.chooseLineColor), this, this.optionManager, new ColorOptionPopUp(OptionChangedEvent.Option.lineColor, this.optionManager.getLineColor()));
        this.toolBar.add(opBut);
        
        // Fill Color
        opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.chooseFillColor), this, this.optionManager, new ColorOptionPopUp(OptionChangedEvent.Option.fillColor, this.optionManager.getFillColor()));
        this.toolBar.add(opBut);
                
        // Text Options (Font, Color, Size, Alignment, Style)
        HashMap<OptionChangedEvent.Option, Object> options = new HashMap<OptionChangedEvent.Option, Object>();
        options.put(OptionChangedEvent.Option.font, this.optionManager.getFont());
        options.put(OptionChangedEvent.Option.textSize, this.optionManager.getTextSize());
        options.put(OptionChangedEvent.Option.textOption, this.optionManager.getTextOption());
        options.put(OptionChangedEvent.Option.textAlignment, this.optionManager.getTextAlignment());
        options.put(OptionChangedEvent.Option.textColor, this.optionManager.getTextColor());
        opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.drawText), this, this.optionManager, new TextOptionPopUp(options));
        this.toolBar.add(opBut);

        // Zoom
        opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.changeZoomFactor), this, this.optionManager, new ZoomOptionPopUp(OptionChangedEvent.Option.zoomFactor, this.optionManager.getZoomFactor()));
        this.toolBar.add(opBut);
        
        // Grid Wideness
        opBut = new ToolBarOptionButton(Reader.getPropertyValueAsString(Reader.StringProperty.changeGridWideness), this, this.optionManager, new GridWidenessOptionPopUp(OptionChangedEvent.Option.gridWideness, this.optionManager.getGridWideness()));
        this.toolBar.add(opBut);

    }
    
    /**
     * @return the editor
     */
    public Editor getEditor() {
        return editor;
    }

    /**
     * @return the activeButton
     */
    public ToolBarButton getActiveButton() {
        return activeButton;
    }

    /**
     * @return the toolBar
     */
    public JToolBar getToolBar() {
        return toolBar;
    }
    
    /**
     * Sets the ToolBarSelectionButton active.
     */
    public void switchToSelectionButton() {
        this.setActiveButton(this.selectionButton);
        this.editor.getCanvas().updateMouse(null);
    }
    
    /**
     * Sets the last ToolBarButton active.
     */
    public void setPreviousButtonActive() {
        if(this.previousButton != null) {
            this.setActiveButton(this.previousButton);
        }
    }
    
    public void setActiveButton (ToolBarButton tbb) {
        if(this.activeButton == tbb)
            return;
        this.previousButton = this.activeButton;
        this.activeButton = tbb;
        this.setButtonFocus();
        this.toolBar.repaint();
    }

    private void setButtonFocus() {
        if(this.previousButton != null)
            this.previousButton.setFocus(false);
        this.activeButton.setFocus(true);
    }

    @Override
    public void gotFocus(ToolBarButton tbb) {
        setActiveButton(tbb);
    }
}
