/**----------------------------------------------------
 * File:      OptionManager.java
 * Package:   at.hoertler.svgeditor.options
 * Project:   SVGEditor
 * ----------------------------------------------------
 * @author Philipp Hörtler BSc
 * @version 1.0.0
 * Date: 2013-03-24
 * ----------------------------------------------------
 * Description:
 * 
 * The OptionManager caches the standard values and 
 * forwards new settings to selected elements.
 */
package at.hoertler.svgeditor.options;

import at.hoertler.svgeditor.data.ElementManager;
import at.hoertler.svgeditor.event.OptionChangedListener;
import at.hoertler.svgeditor.event.OptionChangedEvent;
import at.hoertler.svgeditor.event.OptionChangedEvent.Option;
import at.hoertler.svgeditor.svg.elements.SVGElement;
import java.awt.Color;
import java.awt.Font;
import java.awt.GraphicsEnvironment;
import java.util.ArrayList;

public class OptionManager {

  public enum Alignment {

    left, right, center
  };

  public enum TextOption {

    bold, italic, underlined, strikethrough
  };
  
  private int lineStrength, textSize, gridWideness, zoomFactor;
  private Font font;
  private Color lineColor, fillColor, textColor;
  private Alignment textAlignment;
  private ArrayList<TextOption> textOption;
  private ArrayList<OptionChangedListener> listener = new ArrayList<OptionChangedListener>();
  private ElementManager elementManager = null;

  public void setElementOptions(SVGElement elem) {
    elem.setLineStrength(this.getLineStrength());
    elem.setLineColor(this.getLineColor());
    elem.setFillColor(this.getFillColor());
    elem.setFont(this.getFont());
    elem.setTextColor(this.getTextColor());
    elem.setTextAlignment(this.getTextAlignment());
    elem.setTextOption(this.getTextOption());
    elem.setTextSize(this.getTextSize());
    elem.setText("");
  }

  private void fireOptionChangedEvent(Option o, Object value) {
    for (OptionChangedListener l : listener) {
      l.optionChanged(new OptionChangedEvent(o, value));
    }
  }

  /**
   * This method sets the value for the specified option type.
   *
   * @param o     Option type.
   * @param value Value to set.
   */
  public void setValueForOption(Option o, Object value) {
    switch (o) {
      case lineStrength:
        setLineStrength((int) value);
        break;
      case lineColor:
        setLineColor((Color) value);
        break;
      case fillColor:
        setFillColor((Color) value);
        break;
      case font:
        setFont((Font) value);
        break;
      case text:
        setText((String) value);
        break;
      case textColor:
        setTextColor((Color) value);
        break;
      case textSize:
        setTextSize((int) value);
        break;
      case textAlignment:
        setTextAlignment((Alignment) value);
        break;
      case textOption:
        setTextOptions((ArrayList<TextOption>) value);
        break;
      case zoomFactor:
        setZoomFactor((int) value);
        break;
      case gridWideness:
        setGridWideness((int) value);
        break;
    }
  }

  /**
   * This method gets the value for the specified option type.
   *
   * @param o Option type.
   *
   * @return value as object.
   */
  public Object getValueForOption(Option o) {
    switch (o) {
      case lineStrength:
        return getLineStrength();
      case lineColor:
        return getLineColor();
      case fillColor:
        return getFillColor();
      case font:
        return getFont();
      case textColor:
        return getTextColor();
      case textSize:
        return getTextSize();
      case textAlignment:
        return getTextAlignment();
      case textOption:
        return getTextOption();
      case gridWideness:
        return getGridWideness();
      case zoomFactor:
        return getZoomFactor();
      default:
        return null;
    }
  }

  /**
   * Adds an listener to the OptionManager.
   *
   * @param ocl OptionChangedListener
   */
  public void addOptionChangedListener(OptionChangedListener ocl) {
    listener.add(ocl);
    this.setGridWideness(this.getGridWideness());
  }

  /**
   * Removes an listener from the OptionManager.
   *
   * @param ocl OptionChangedListener
   */
  public void removeOptionChangedListener(OptionChangedListener ocl) {
    listener.remove(ocl);
  }

  /**
   * @return the lineStrength
   */
  public int getLineStrength() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getLineStrength();
    }
    return lineStrength;
  }

  /**
   * @param aLineStrength the lineStrength to set
   */
  public void setLineStrength(int aLineStrength) {
    lineStrength = aLineStrength;
    fireOptionChangedEvent(Option.lineStrength, lineStrength);
  }

  /**
   * @param text the text to set
   */
  public void setText(String text) {
    fireOptionChangedEvent(Option.text, text);
  }

  /**
   * @return the textSize
   */
  public int getTextSize() {
    if (this.elementManager==null || !this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getTextSize();
    }
    return textSize;
  }

  /**
   * @param aTextSize the textSize to set
   */
  public void setTextSize(int aTextSize) {
    textSize = aTextSize;
    fireOptionChangedEvent(Option.textSize, textSize);
  }

  /**
   * @return the lineColor
   */
  public Color getLineColor() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getLineColor();
    }
    return lineColor;
  }

  /**
   * @param aLineColor the lineColor to set
   */
  public void setLineColor(Color aLineColor) {
    lineColor = aLineColor;
    fireOptionChangedEvent(Option.lineColor, lineColor);
  }

  /**
   * @return the fillColor
   */
  public Color getFillColor() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getFillColor();
    }
    return fillColor;
  }

  /**
   * @param aFillColor the fillColor to set
   */
  public void setFillColor(Color aFillColor) {
    fillColor = aFillColor;
    fireOptionChangedEvent(Option.fillColor, fillColor);
  }

  /**
   * @return the textColor
   */
  public Color getTextColor() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getTextColor();
    }
    return textColor;
  }

  /**
   * @param aTextColor the textColor to set
   */
  public void setTextColor(Color aTextColor) {
    textColor = aTextColor;
    fireOptionChangedEvent(Option.textColor, textColor);
  }

  /**
   * @return the font
   */
  public Font getFont() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getFont();
    }
    return font;
  }

  /**
   * @param aFont the font to set
   */
  public void setFont(Font aFont) {
    font = aFont;
    fireOptionChangedEvent(Option.font, aFont);
  }

  /**
   * @return the textAlignment
   */
  public Alignment getTextAlignment() {
    if (!this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getTextAlignment();
    }
    return textAlignment;
  }

  /**
   * @param aTextAlignment the textAlignment to set
   */
  public void setTextAlignment(Alignment aTextAlignment) {
    textAlignment = aTextAlignment;
    fireOptionChangedEvent(Option.textAlignment, textAlignment);
  }

  /**
   * @return the textOption
   */
  public ArrayList<TextOption> getTextOption() {
    if (this.elementManager!=null && !this.elementManager.getSelectedElements().isEmpty()) {
      return this.elementManager.getSelectedElements().get(0).getTextOption();
    }
    return textOption;
  }

  /**
   * @param aTextOption the textOption to set
   */
  public void setTextOption(TextOption aTextOption) {
    if (!textOption.contains(aTextOption)) {
      textOption.add(aTextOption);
      fireOptionChangedEvent(Option.textOption, textOption);
    }
  }

  /**
   * @param aTextOption the textOption to set
   */
  public void setTextOptions(ArrayList<TextOption> aTextOption) {
    textOption = aTextOption;
    fireOptionChangedEvent(Option.textOption, textOption);
  }

  /**
   * @param aTextOption the textOption to remove
   */
  public void removeTextOption(TextOption aTextOption) {
    if (textOption.contains(aTextOption)) {
      textOption.remove(aTextOption);
      fireOptionChangedEvent(Option.textOption, textOption);
    }
  }

  /**
   * Clears all text options
   */
  public void clearTextOptions() {
    textOption.clear();
  }

  /**
   * @return the zoomFactor
   */
  public int getZoomFactor() {
    return zoomFactor;
  }

  /**
   * @param zoomFactor the zoomFactor to set
   */
  public void setZoomFactor(int zoomFactor) {
    if (zoomFactor > 0) {
      this.zoomFactor = zoomFactor;
      fireOptionChangedEvent(Option.zoomFactor, zoomFactor);
    }
  }

  /**
   * @param elementManager the elementManager to set
   */
  public void setElementManager(ElementManager elementManager) {
    this.elementManager = elementManager;
    
  }

  /**
   * @return the gridWideness
   */
  public int getGridWideness() {
    return gridWideness;
  }

  /**
   * @param aGridWideness the gridWideness to set
   */
  public void setGridWideness(int aGridWideness) {
    gridWideness = aGridWideness;
    fireOptionChangedEvent(Option.gridWideness, gridWideness);
  }

  private void init() {
    this.textOption = new ArrayList<TextOption>();
    this.setLineStrength(1);
    this.setLineColor(Color.BLACK);
    this.setFillColor(new Color(255, 255, 255, 0));
    this.setGridWideness(4);
    this.setTextAlignment(Alignment.left);
    this.setTextColor(Color.BLACK);
    this.
            setFont(GraphicsEnvironment.getLocalGraphicsEnvironment().
            getAllFonts()[0]);
    this.setTextSize(16);
    this.clearTextOptions();
    this.setZoomFactor(1);
    this.setText("");
  }
  
  public OptionManager() {
    this.init();
  }
}
